# -*- coding: utf-8 -*-
# This file is part of ekd - Music and sound module
# Copyright (C) 2007-2009  Olivier Ponchaut <opvg@edpnet.be>

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import os
import subprocess
from PyQt4.QtCore import QObject, SIGNAL, QString, QThread
from gui_modules_common.EkdWidgets import EkdProgress
from moteur_modules_common.EkdConfig import EkdConfig
from moteur_modules_common.EkdProcess import EkdProcess
from moteur_modules_common.EkdCompatiblePython2Et3 import EkdPrint

# Activation des traces de débug sur ce moteur -> mettre à 1 ou True.
debug = 1

class soxProcessMulti(EkdProgress) :

	def __init__(self, cheminAudioSource, cheminAudioSorti, acodec=u"", ar=48000, ab=128000, parent=None) :
		super(soxProcessMulti, self).__init__(parent, 100)
		if debug : EkdPrint(unicode(cheminAudioSource))
		self.processAudio = soxEngineMulti(cheminAudioSource, cheminAudioSorti, acodec, ar, ab, parent)
		self.connect(self.processAudio, SIGNAL("DisplayText"), self.addText)
		self.connect(self.processAudio, SIGNAL("Progress"), self.setProgress)
		self.connect(self.processAudio, SIGNAL("endProcess(QString)"), self.endAction)

	def setProgress(self, prg) :
		self.barProgress.setValue(prg)

	def endAction(self, msg) :
		self.barProgress.setValue(100)
		self.fermer.setEnabled(True)
		self.emit(SIGNAL("endProcess"), msg)

	def run(self) :
		self.processAudio.start()


class soxEngineMulti(QThread) :

	def __init__(self, cheminIn, cheminOut, acodec, ar, ab, parent=None) :
		super(soxEngineMulti, self).__init__(parent)
		self.cheminIn = cheminIn
		self.cheminOut = cheminOut
		self.acodec=acodec
		self.ar=ar
		self.ab=ab
		self.k=0
				

	def run(self):
		if debug : EkdPrint(unicode(self.cheminIn))
		urlFFmpeg=EkdConfig.urlFFmpeg
		urlSox=EkdConfig.urlSox
		# Vérification que le répertoire temporaire existe et est vide.
		if os.path.exists(EkdConfig.getTempDir() + os.sep+ u'concat_audio' + os.sep) :
			for tmpFile in os.listdir(EkdConfig.getTempDir() + os.sep+ u'concat_audio' + os.sep) :
				os.remove(EkdConfig.getTempDir() + os.sep+ u'concat_audio' + os.sep + tmpFile)
		else : os.mkdir(EkdConfig.getTempDir() + os.sep+ u'concat_audio' + os.sep)

		entree=u""
		self.k=0
		for fileA in self.cheminIn :
			### Convertir tous les fichiers de la liste en fichier .wav avec les caractéristiques du fichier de sortie
			com = u"%sffmpeg -y -i \"%s\" -ac 2 -ar %d  \"%stmp_%d_.wav\"" % ( \
							urlFFmpeg, \
							fileA, \
							int(self.ar), \
							EkdConfig.getTempDir() + os.sep+ u'concat_audio' + os.sep, \
							self.k)
			# Debug
			if debug : EkdPrint(com)
			self.mesg2 = EkdProcess(com.encode(EkdConfig.coding), outputerr = subprocess.STDOUT)
			while self.mesg2.poll() == None :
				prog = self.mesg2.stdout.readline()
				self.emit(SIGNAL("DisplayText"), prog)
			entree += u" -t wav \"%stmp_%i_.wav\"" % (unicode(EkdConfig.getTempDir() + os.sep+ u'concat_audio' + os.sep),self.k)
			self.emit(SIGNAL("DisplayText"),_(u"Réencodage du fichier %s terminé") % (fileA))
			self.k += 1
			self.emit(SIGNAL("Progress"), int(self.k*100.0/(len(self.cheminIn)+1)))

			# Debug
		if debug : EkdPrint(entree)
		commande = u"%ssox -S --combine concatenate %s -t wav -c 2 -r %d -b 16 --endian little -e signed-integer - | \"%sffmpeg\" -y -acodec pcm_s16le -ar %d -ac 2 -f wav -i - %s -ac 2 -ar %d -ab %s \"%s\"" % ( \
							urlSox, \
							entree, \
							int(self.ar), \
							urlFFmpeg, \
							int(self.ar), \
							self.acodec, \
							int(self.ar), \
							self.ab, \
							self.cheminOut)
		### Debug
		if debug : EkdPrint(commande)
		self.mesg = EkdProcess(commande.encode(EkdConfig.coding), \
							outputerr=subprocess.STDOUT)
		nfile = -1
		while self.mesg.poll() == None :
			prog = self.mesg.stdout.readline()
			self.emit(SIGNAL("DisplayText"), prog)

		self.finEncodage(self.mesg.wait())

	def finEncodage(self, statutDeSortie):
		"""choses à faire à la fin de l'encodage audio"""
		if statutDeSortie==1:
			EkdPrint(u"Crash de l'encodeur!")
			self.emit(SIGNAL("DisplayText"),_(u"Crash de l'encodeur !"))
			self.emit(SIGNAL("endProcess(QString)"), QString(u"None"))

		elif statutDeSortie==2 :
			EkdPrint(u"Problème lors de l'encodage audio final : %s" % statutDeSortie)
			self.emit(SIGNAL("endProcess(QString)"), None)
			self.emit(SIGNAL("DisplayText"),_(u"Le traitement ne s'est pas déroulé correctement."))

		else :
			EkdPrint(u"Fin et statut : %s" % statutDeSortie)
			self.emit(SIGNAL("endProcess(QString)"), self.cheminOut)
			self.emit(SIGNAL("DisplayText"),_(u"Encodage terminé avec succès"))
		for i in range(self.k) :
			os.remove(u"%stmp_%i_.wav" % (unicode(EkdConfig.getTempDir() + os.sep+ u'concat_audio' + os.sep),i))
			i += 1